<!DOCTYPE kpartplugin>
<kpartplugin library="foobar">
<MenuBar>
 <Menu name="file"><Text>&amp;File</Text>
  <Action name="searchreplace"/>
 </Menu>
 <Menu name="woohoo"><Text>fun</Text>
  <Action name="searchreplace"/>
 </Menu>
</MenuBar>
<ToolBar name="gah"/>
</kpartplugin>
