/* This file is part of the KDE project
   Copyright (C) 1999 Simon Hausmann <hausmann@kde.org>
             (C) 1999 David Faure <faure@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/
#ifndef __kparts_event_h__
#define __kparts_event_h__

#include <kparts/kparts_export.h>

#include <QEvent>

class QWidget;

namespace KParts
{

class EventPrivate;
/**
 * @class Event event.h <KParts/Event>
 *
 * @short Base class for all KParts events.
 */
class KPARTS_EXPORT Event : public QEvent
{
public:
    Event(const char *eventName);
    virtual ~Event();
    virtual const char *eventName() const;

    static bool test(const QEvent *event);
    static bool test(const QEvent *event, const char *name);

private:
    EventPrivate *const d;
};

} // namespace

#endif
