<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="KGVShell" version="0">
<MenuBar>
 <Menu name="file"><text>&amp;File</text>
  <Action name="file_open"/>
  <Separator />
  <ActionList name="file_openwith"/> <!-- includes termination separator -->
  <Action name="file_quit"/>
 </Menu>
</MenuBar>
<ToolBar name="KGV-ToolBar">
 <Action name="file_open"/>
 <Action name="file_quit"/>
</ToolBar>
</gui>

