<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="Shell" version="4">
<MenuBar>
  <Menu name="file"><text>&amp;File</text>
   <Action name="open_local_file"/>
   <Action name="open_remote_file"/>
   <Separator/>
   <Action name="edit_file"/>
   <Action name="close_editor"/>
   <Merge name="NotepadPart"/>
   <Separator/>
   <Merge/>
   <Separator/>
   <Action name="shell_quit"/>
  </Menu>
  <Merge/>
  <Menu name="blah"><text>BlahFoo</text>
   <Action name="shell_yami"/>
   <Menu name="nonsense"><text>Dummdidumm</text>
    <Action name="shell_yasmi"/>
   </Menu>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar">
 <Action name="shell_quit"/>
</ToolBar>
</gui>
